import * as i0 from '@angular/core';
import { Injectable, Inject, EventEmitter, Directive, Input, Output, Component, ViewEncapsulation, ChangeDetectionStrategy, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { DOCUMENT, CommonModule } from '@angular/common';

class AbmConfig {
}

class LoaderService {
    constructor(cog, doc) {
        this.doc = doc;
        this._cog = Object.assign({ apiProtocol: 'auto', apiVersion: '3.0', apiCallback: 'angularBaiduMapsLoader', apiHostAndPath: 'api.map.baidu.com/api' }, cog);
    }
    load() {
        if (this._scriptLoadingPromise) {
            return this._scriptLoadingPromise;
        }
        this._scriptLoadingPromise = new Promise((resolve, reject) => {
            this.loadScript(this._getSrc(), this._cog.apiCallback)
                .then(() => {
                const libs = this._cog.libraries || [];
                if (libs.length === 0) {
                    return Promise.resolve();
                }
                return Promise.all(libs.map((src) => this.loadScript(src))).then(() => Promise.resolve());
            })
                .then(resolve)
                .catch(reject);
        });
        return this._scriptLoadingPromise;
    }
    loadScript(src, callback) {
        const script = this.doc.createElement('script');
        script.type = 'text/javascript';
        script.async = true;
        script.defer = true;
        script.src = src;
        const res = new Promise((resolve, reject) => {
            script.onload = () => {
                if (callback) {
                    window[callback] = () => {
                        resolve();
                    };
                }
                else {
                    resolve();
                }
            };
            script.onerror = (error) => {
                reject(error);
            };
        });
        this.doc.body.appendChild(script);
        return res;
    }
    _getSrc() {
        let protocol;
        const { apiProtocol, apiVersion, apiKey, apiCallback, apiHostAndPath } = this._cog;
        switch (apiProtocol) {
            case 'http':
                protocol = 'http:';
                break;
            case 'https':
                protocol = 'https:';
                break;
            case 'file':
                protocol = 'file:';
                break;
            default:
                protocol = '';
                break;
        }
        const queryParams = {
            v: apiVersion,
            ak: apiKey,
            callback: apiCallback,
        };
        const params = Object.keys(queryParams)
            .filter((k) => queryParams[k] != null)
            .filter((k) => {
            return !Array.isArray(queryParams[k]) || (Array.isArray(queryParams[k]) && queryParams[k].length > 0);
        })
            .map((k) => {
            const i = queryParams[k];
            if (Array.isArray(i)) {
                return { key: k, value: i.join(',') };
            }
            return { key: k, value: i };
        })
            .map((entry) => {
            return `${entry.key}=${entry.value}`;
        })
            .join('&');
        return `${protocol}//${apiHostAndPath}?${params}`;
    }
}
LoaderService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: LoaderService, deps: [{ token: AbmConfig }, { token: DOCUMENT }], target: i0.ɵɵFactoryTarget.Injectable });
LoaderService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: LoaderService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: LoaderService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: AbmConfig }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }]; } });

class AbmBaseComponent {
    constructor(el, COG, zone, loader) {
        this.el = el;
        this.COG = COG;
        this.zone = zone;
        this.loader = loader;
        this.destroy$ = new Subject();
        this.ready = new EventEmitter();
        this.config = Object.assign({ apiProtocol: 'auto', apiVersion: '3.0', apiCallback: 'angularBaiduMapsLoader', apiHostAndPath: 'api.map.baidu.com/api' }, COG);
    }
    _updateOptions() {
        this.options = Object.assign(Object.assign({}, this.defaultOptions()), this.options);
        const map = this.map;
        if (map && map.setOptions) {
            map.setOptions(this.options);
        }
    }
    _initMap() {
        if (this.map) {
            return;
        }
        this.loader
            .load()
            .then(() => {
            this.zone.runOutsideAngular(() => this.create());
            this.ready.emit(this.map);
        })
            .catch((error) => {
            console.warn('js加载失败', error);
        });
    }
    _getSrc() {
        let protocol;
        const { apiProtocol, apiHostAndPath, apiVersion, apiKey, apiCallback } = this.config;
        switch (apiProtocol) {
            case 'http':
                protocol = 'http:';
                break;
            case 'https':
                protocol = 'https:';
                break;
            case 'file':
                protocol = 'file:';
                break;
            default:
                protocol = '';
                break;
        }
        const queryParams = {
            v: apiVersion,
            ak: apiKey,
            callback: apiCallback,
        };
        const params = Object.keys(queryParams)
            .filter((k) => queryParams[k] != null)
            .filter((k) => {
            return !Array.isArray(queryParams[k]) || (Array.isArray(queryParams[k]) && queryParams[k].length > 0);
        })
            .map((k) => {
            const i = queryParams[k];
            if (Array.isArray(i)) {
                return { key: k, value: i.join(',') };
            }
            return { key: k, value: i };
        })
            .map((entry) => {
            return `${entry.key}=${entry.value}`;
        })
            .join('&');
        return `${protocol}//${apiHostAndPath}?${params}`;
    }
    ngOnInit() {
        if (!(typeof document === 'object' && !!document)) {
            return;
        }
        this._initMap();
    }
    ngOnChanges(changes) {
        if ('options' in changes) {
            this._updateOptions();
        }
    }
    ngOnDestroy() {
        this.destroy();
        this.destroy$.next();
        this.destroy$.complete();
    }
}
AbmBaseComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmBaseComponent, deps: [{ token: i0.ElementRef }, { token: AbmConfig }, { token: i0.NgZone }, { token: LoaderService }], target: i0.ɵɵFactoryTarget.Directive });
AbmBaseComponent.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.2", type: AbmBaseComponent, inputs: { options: "options" }, outputs: { ready: "ready" }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmBaseComponent, decorators: [{
            type: Directive
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: AbmConfig }, { type: i0.NgZone }, { type: LoaderService }]; }, propDecorators: { options: [{
                type: Input
            }], ready: [{
                type: Output
            }] } });

class AbmComponent extends AbmBaseComponent {
    defaultOptions() {
        return this.COG.mapOptions;
    }
    create() {
        try {
            this.map = new BMap.Map(this.el.nativeElement, this.options);
        }
        catch (ex) {
            console.warn('地图初始化失败', ex);
        }
    }
    destroy() {
        if (this.map) {
            this.map.clearOverlays();
            this.map.clearHotspots();
        }
    }
}
AbmComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AbmComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.2", type: AbmComponent, selector: "abm-map, [abm-map]", host: { properties: { "class.angular-baidu-maps-container": "true" } }, exportAs: ["abmMap"], usesInheritance: true, ngImport: i0, template: ``, isInline: true, styles: ["\n      .angular-baidu-maps-container {\n        display: block;\n        width: 100%;\n        height: 100%;\n      }\n    "], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'abm-map, [abm-map]',
                    exportAs: 'abmMap',
                    template: ``,
                    styles: [
                        `
      .angular-baidu-maps-container {
        display: block;
        width: 100%;
        height: 100%;
      }
    `,
                    ],
                    host: {
                        '[class.angular-baidu-maps-container]': 'true',
                    },
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }] });

class AbmPanoramaComponent extends AbmBaseComponent {
    defaultOptions() {
        return this.COG.panoramaOptions;
    }
    create() {
        try {
            this.map = new BMap.Panorama(this.el.nativeElement, this.options);
        }
        catch (ex) {
            console.warn('全景初始化失败', ex);
        }
    }
    destroy() { }
}
AbmPanoramaComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmPanoramaComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AbmPanoramaComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.2", type: AbmPanoramaComponent, selector: "abm-panorama, [abm-panorama]", host: { properties: { "class.angular-baidu-maps-container": "true" } }, exportAs: ["abmPanoramaMap"], usesInheritance: true, ngImport: i0, template: ``, isInline: true, styles: ["\n      .angular-baidu-maps-container {\n        display: block;\n        width: 100%;\n        height: 100%;\n      }\n    "], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmPanoramaComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'abm-panorama, [abm-panorama]',
                    exportAs: 'abmPanoramaMap',
                    template: ``,
                    styles: [
                        `
      .angular-baidu-maps-container {
        display: block;
        width: 100%;
        height: 100%;
      }
    `,
                    ],
                    host: {
                        '[class.angular-baidu-maps-container]': 'true',
                    },
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }] });

class AbmModule {
    static forRoot(config) {
        return {
            ngModule: AbmModule,
            providers: [{ provide: AbmConfig, useValue: config }],
        };
    }
}
AbmModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
AbmModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmModule, declarations: [AbmComponent, AbmPanoramaComponent], imports: [CommonModule], exports: [AbmComponent, AbmPanoramaComponent] });
AbmModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmModule, providers: [LoaderService], imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.2", ngImport: i0, type: AbmModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    providers: [LoaderService],
                    declarations: [AbmComponent, AbmPanoramaComponent],
                    exports: [AbmComponent, AbmPanoramaComponent],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { AbmComponent, AbmConfig, AbmModule, AbmPanoramaComponent };
//# sourceMappingURL=angular-baidu-maps.js.map
